@@ -99,7 +99,104 @@ Bin = override(Bin)
 __all__.append('Bin')
 
 
-class Caps(Gst.Caps):
+class NotWritableMiniObject(Exception):
+    pass
+
+
+__all__.append('NotWritableMiniObject')
+
+
+class MiniObject:
+    def make_writable(self):
+        return _gi_gst.mini_object_make_writable(self)
+
+    def is_writable(self):
+        return _gi_gst.mini_object_is_writable(self)
+
+    @property
+    def flags(self):
+        return _gi_gst.mini_object_flags(self)
+
+    @flags.setter
+    def flags(self, flags):
+        _gi_gst.mini_object_set_flags(self, flags)
+
+
+class NotWritableQuery(Exception):
+    pass
+
+
+__all__.append('NotWritableQuery')
+
+
+class Query(Gst.Query, MiniObject):
+    def get_structure(self):
+        return StructureWrapper(_gi_gst.query_get_structure(self), self,
+                                False)
+
+    def writable_structure(self):
+        return StructureWrapper(_gi_gst.query_writable_structure(self), self, True)
+
+
+Query = override(Query)
+__all__.append('Query')
+
+
+class NotWritableEvent(Exception):
+    pass
+
+
+__all__.append('NotWritableEvent')
+
+
+class Event(Gst.Event, MiniObject):
+    def get_structure(self):
+        return StructureWrapper(_gi_gst.event_get_structure(self), self,
+                                False)
+
+    def writable_structure(self):
+        return StructureWrapper(_gi_gst.event_writable_structure(self), self, True)
+
+
+Event = override(Event)
+__all__.append('Event')
+
+
+class NotWritableContext(Exception):
+    pass
+
+
+__all__.append('NotWritableContext')
+
+
+class Context(Gst.Context, MiniObject):
+    def get_structure(self):
+        return StructureWrapper(_gi_gst.context_get_structure(self), self,
+                                False)
+
+    def writable_structure(self):
+        return StructureWrapper(_gi_gst.context_writable_structure(self), self, True)
+
+
+Context = override(Context)
+__all__.append('Context')
+
+
+class NotWritableCaps(Exception):
+    pass
+
+
+__all__.append('NotWritableCaps')
+
+
+class NotWritableStructure(Exception):
+    pass
+
+
+__all__.append('NotWritableStructure')
+
+
+class Caps(Gst.Caps, MiniObject):
 
     def __nonzero__(self):
         return not self.is_empty()
@@ -107,9 +204,11 @@ class Caps(Gst.Caps):
     def __new__(cls, *args):
         if not args:
             return Caps.new_empty()
-        elif len(args) > 1:
+        if len(args) > 1:
             raise TypeError("wrong arguments when creating GstCaps object")
-        elif isinstance(args[0], str):
+
+        assert len(args) == 1
+        if isinstance(args[0], str):
             return Caps.from_string(args[0])
         elif isinstance(args[0], Caps):
             return args[0].copy()
@@ -134,11 +233,19 @@ class Caps(Gst.Caps):
     def __getitem__(self, index):
         if index >= self.get_size():
             raise IndexError('structure index out of range')
-        return self.get_structure(index)
+
+        return Gst.Caps.get_structure(self, index)
 
     def __len__(self):
         return self.get_size()
 
+    def get_structure(self, index):
+        return StructureWrapper(_gi_gst.caps_get_structure(self, index), self,
+                                False)
+
+    def writable_structure(self, index):
+        return StructureWrapper(_gi_gst.caps_writable_structure(self, index), self, True)
+
 
 Caps = override(Caps)
 __all__.append('Caps')
@@ -307,25 +414,31 @@ class Structure(Gst.Structure):
             if kwargs:
                 raise TypeError("wrong arguments when creating GstStructure, first argument"
                                 " must be the structure name.")
-            return Structure.new_empty()
+            struct = Structure.new_empty()
+            return struct
         elif len(args) > 1:
             raise TypeError("wrong arguments when creating GstStructure object")
         elif isinstance(args[0], str):
             if not kwargs:
-                return Structure.from_string(args[0])[0]
+                struct = Structure.from_string(args[0])[0]
+                return struct
             struct = Structure.new_empty(args[0])
             for k, v in kwargs.items():
                 struct[k] = v
 
             return struct
         elif isinstance(args[0], Structure):
-            return args[0].copy()
+            struct = args[0].copy()
+            return struct
 
         raise TypeError("wrong arguments when creating GstStructure object")
 
     def __init__(self, *args, **kwargs):
         pass
 
+    def __ptr__(self):
+        return _gi_gst._get_object_ptr(self)
+
     def __getitem__(self, key):
         return self.get_value(key)
 
@@ -342,6 +455,14 @@ class Structure(Gst.Structure):
     def __setitem__(self, key, value):
         return self.set_value(key, value)
 
+    def set_value(self, key, value):
+        if not _gi_gst.structure_is_writable(self):
+            raise NotWritableStructure("Trying to write to a not writable structure."
+                                       " Make sure to use the right APIs to have access to structure"
+                                       " in a writable way.")
+
+        return Gst.Structure.set_value(self, key, value)
+
     def __str__(self):
         return self.to_string()
 
@@ -682,6 +803,19 @@ def pairwise(iterable):
     return zip(a, b)
 
 
+class StructureWrapper:
+    def __init__(self, structure, parent, writable):
+        self.__structure = structure
+        self.__parent__ = parent
+
+    def __enter__(self):
+        return self.__structure
+
+    def __exit__(self, _type, _value, _tb):
+        self.__parent__ = False
+        return
+
+
 class MapInfo:
     def __init__(self):
         self.memory = None
@@ -712,7 +846,54 @@ class MapInfo:
 __all__.append("MapInfo")
 
 
-class Buffer(Gst.Buffer):
+class Buffer(Gst.Buffer, MiniObject):
+    @property
+    def flags(self):
+        return _gi_gst.mini_object_flags(self)
+
+    @flags.setter
+    def flags(self, flags):
+        _gi_gst.mini_object_set_flags(self, flags)
+
+    @property
+    def dts(self):
+        return _gi_gst.buffer_get_dts(self)
+
+    @dts.setter
+    def dts(self, dts):
+        _gi_gst.buffer_set_dts(self, dts)
+
+    @property
+    def pts(self):
+        return _gi_gst.buffer_get_pts(self)
+
+    @pts.setter
+    def pts(self, pts):
+        _gi_gst.buffer_set_pts(self, pts)
+
+    @property
+    def duration(self):
+        return _gi_gst.buffer_get_duration(self)
+
+    @duration.setter
+    def duration(self, duration):
+        _gi_gst.buffer_set_duration(self, duration)
+
+    @property
+    def offset(self):
+        return _gi_gst.buffer_get_offset(self)
+
+    @offset.setter
+    def offset(self, offset):
+        _gi_gst.buffer_set_offset(self, offset)
+
+    @property
+    def offset_end(self):
+        return _gi_gst.buffer_get_offset_end(self)
+
+    @offset_end.setter
+    def offset_end(self, offset_end):
+        _gi_gst.buffer_set_offset_end(self, offset_end)
 
     def map_range(self, idx, length, flags):
         mapinfo = MapInfo()
